/*
* Copyright (C) Mellanox Technologies, Ltd. 2001-2014. ALL RIGHTS RESERVED.
*
* Feb 25, 2014
* This software product is a proprietary product of Mellanox Technologies, Ltd.
* (the "Company") and all right, title, and interest in and to the software product,
* including all associated intellectual property rights, are and shall
* remain exclusively with the Company.
*
* This software product is governed by the End User License Agreement
* provided with the software product.
*
*/

#ifndef __MLNX_IF_US_H_
#define __MLNX_IF_US_H_

#ifndef I2C_NAME_SIZE
#define I2C_NAME_SIZE   20
#endif

#include <sys/ioctl.h>
#include <linux/i2c-dev.h>
#include <linux/i2c.h>
#include "mlnx_if.h"

#ifndef __KERNEL__
/* keep them for compatibility */
typedef short s16;
typedef unsigned short u16;
typedef int s32;
typedef unsigned int u32;
typedef char s8;
typedef unsigned char u8;
typedef unsigned long u64;
#endif

/* 
  Description: sending i2c read request to devices.
  Input:
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_read(int bus, u16 addr, u32 offset, size_t addr_width, char* data, size_t datalen);

/* 
  Description: sending i2c write request to devices.
  Input:
               bus        - bus id to which write request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start write request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_write(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen);

/* 
  Description: sending i2c read request to devices.
  Input:
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_read_vpd(int bus, u16 addr, u32 offset, size_t addr_width, char* data, size_t datalen);

/* 
  Description: sending i2c read request to devices to read configuration register space .
  Input:
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_read_crspace(int bus, u16 addr, u32 offset, char* data, size_t datalen);

/* 
  Description: sending i2c write request to devices.
               For some diveces (f.e. ATMEL EEPROM) family memory internally organized 
               as the pages of 128-bytes each. If more than 128 data words are transmitted
               to such device, the data word address rolls over and previous data is 
               getting overwritten. API will divide transaction to separate chunks in case
               ((offset % I2C_CHUNK_MAX) + datalen) > I2C_CHUNK_MAX
  Input:
               bus        - bus id to which write request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start write request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_write_vpd(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen);

/* 
  Description: sending i2c read request to devices.
               For some diveces (f.e. ATMEL EEPROM) family memory internally organized 
               as the pages of 128-bytes each. If more than 128 data words are transmitted
               to such device, the data word address rolls over and previous data is 
               getting overwritten. API will divide transaction to separate chunks in case
               ((offset % I2C_CHUNK_MAX) + datalen) > I2C_CHUNK_MAX
  Input:
               attr       - attribute from /sys/bus/$BUS-$ADDR/attr, which can be accessed
                            through sysfs
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_read_attr(const char *attr, int bus, int addr, int offset, int length, char* data);

/* 
  Description: sending i2c read request to devices, while request contains some initial data.
               This data is cached, before read request execution.
  Input:
               attr       - attribute from /sys/bus/$BUS-$ADDR/attr, which can be accessed
                            through sysfs
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated, filled out with the initial data and released by user.
*/
int mlnx_i2c_read_attr_cached(const char *attr, int bus, int addr, int offset, int length, char* data);

/* 
  Description: sending i2c write request to devices.
  Input:
               attr       - attribute from /sys/bus/$BUS-$ADDR/attr, which can be accessed
                            through sysfs
               bus        - bus id to which write request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start write request
                            (actually, register address within i2c device);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
               positive - number of bytes have been read
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_write_attr(const char *attr, int bus, int addr, int offset, int length, char* data);

/* 
  Description: adding new virtual buses into i2c tree.
               Actually it's used to introduce one or several switches into the 
               system. Each switch has upper edge bus id to which it should be
               connected to, and LEGS_NUM number of its legs to which could be 
               connected lower edge i2c devices or other swithces.
               Could be added up to MAX_SW_INREQ_NUM number of switches within
               single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_i2c_virtual_buses(struct mlnx_i2c_virt_bus_ioctl_params *params);

/* 
  Description: removing virtual buses into i2c tree.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: blocking exitsing virtual buses of i2c tree.
               After blocking any attempt to send a message through blocked
               bus will be rejected with EBUSY error code.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_block_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: unblocking previously blocked virtual buses of i2c tree.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_unblock_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: unblockin if were blocking and reset virtual buses and attached devices.
               Up to MAX_DEV_INREQ_NUM number of devices could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_reset_i2c_virtual_buses_and_devices(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: adding i2c devices to switch leg.
               Up to MAX_DEV_INREQ_NUM number of devices could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_devices_to_vbus(struct mlnx_i2c_devices_ioctl_params *params);

/* 
  Description: deleting i2c devices from switch leg.
               Up to MAX_DEV_INREQ_NUM number of devices could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_devices_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_devices_from_vbus(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: test existance of virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_scan_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_test_vbus_exist(struct mlnx_i2c_virt_bus_scan_ioctl_params *params);

/* 
  Description: test existance of virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_test_device_exist(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: scanning i2c devices behind i2c switch (directly attached only).
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_scan_switch_devices(struct mlnx_i2c_virt_bus_scan_ioctl_params *params);

/* 
  Description: adding i2c gpio virtual buses.
               Up to MAX_I2C_GPIO_INREQ_NUM number of buses could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_gpio_i2c_buses(struct mlnx_gpio_i2c_ioctl_params *params);

/* 
  Description: deleting i2c gpio virtual buses.
               Up to MAX_I2C_GPIO_INREQ_NUM number of devices could be de-attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_del_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_del_gpio_i2c_buses(struct mlnx_gpio_i2c_del_ioctl_params *params);

/* 
  Description: read gpio pin.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_del_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_gpio_read(struct mlnx_gpio_ioctl_params *params);

/* 
  Description: write gpio pin.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_del_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_gpio_write(struct mlnx_gpio_ioctl_params *params);

/* 
  Description: set gpio pin direction.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_del_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_gpio_set_dir(struct mlnx_gpio_ioctl_params *params);

/* 
  Description: adding single i2c device to virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_one_device_to_vbus(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: deleting single i2c device from virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_one_device_from_vbus(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: subscribe action function into CPLD interrupt handler .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_subscribe_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_subscribe(struct mlnx_cpld_subscribe_ioctl_params *params);

/* 
  Description: unsubscribe action function from CPLD interrupt handler .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_subscribe_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_unsubscribe(struct mlnx_cpld_subscribe_ioctl_params *params);

/* 
  Description: set CPLD interrupt registers layout .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_set_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_set(struct mlnx_cpld_set_ioctl_params *params);

/* 
  Description: get CPLD interrupt registers content .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_get_data_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_get_data(struct mlnx_cpld_get_data_ioctl_params *params);

/* 
  Description: configure system watchdogs
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_watchdog_config_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_watchdog_config(struct mlnx_watchdog_config_ioctl_params *params);

/* 
  Description: enable or disable system watchdogs
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_watchdog_enable_disable_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_watchdog_enable_disable(struct mlnx_watchdog_enable_disable_ioctl_params *params);

/* 
  Description: Configure leds controled by CPLD
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_leds_config_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_leds_config(struct mlnx_cpld_leds_config_ioctl_params *params);

/* 
  Description: Configure fans controled by ASIC
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_asic_fan_config_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_asic_fan_config(struct mlnx_asic_fan_config_ioctl_params *params);

/* 
  Description: Configure temperature sensors controled by ASIC
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_asic_temp_sensor_config_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_asic_temp_config(struct mlnx_asic_temp_sensor_config_ioctl_params *params);

/* 
  Description: sending smbus legacy read request to devices.
  Input:
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_read_smbus_legacy(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen);

/* 
  Description: sending smbus legacy write request to devices.
  Input:
               bus        - bus id to which write request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start write request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_i2c_write_smbus_legacy(int bus, u16 addr, u32 offset, size_t addr_width, char *data, size_t datalen);

int eeprom_rw(int adapter_num, int i2c_addr, unsigned short offset, unsigned short data_size, int eeprom_size);

/*
  Description: sending i2c read and write requests to VPD devices using time limit.
  Input:
               *params - pointer to input template (see description of
                         struct mlnx_rdwr_vpd_i2c_ioctl_params from module mlnx_if.h,
                         where fields are as following:
               bus        - bus id to which read request is to be sent;
	       addr       - i2c device slave address;
	       offset     - offset within i2c device from which to start read request
                            (actually, register address within i2c device);
	       addr_width - width of address (actually, register address size, could be fro 1 to 4 bytes);
	       data       - pointer to buffer where to put data;
	       len        - length of data is to be read or written;
	       bus_prot   - bus, which is used for access protection (set -1 if not required protection);
	       addr_prot  - device address, which is used for access protection;
	       reg_prot   - device register offset, which is used for access protection;
	       get_prot   - value be written to get protection;
	       put_prot   - value be written to put protection;
	       limit      - access limit in msec (set 0 if no limit is required);
	       border     - border limit, which could not be crossing by one transaction (set 0 if no limit is required);
	       rchunk_del - delay in msec between chunk reading (set 0 if no delay is required);
           wchunk_del - delay in msec between chunk writing (set 0 if no delay is required);
           post_del   - delay in msec before release protected bus (set 0 if no delay is required);
	       flags      - access flag - 0 for write, 1 for read
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_vpd_rw_limited(struct mlnx_rdwr_vpd_i2c_ioctl_params *params);

/* 
  Description: sending LPC read or write request to LPC devices.
  Input:
               base       - base address of LPC bus;
	       offset     - offset within LPC device from which to start read or write request 
	       read_write - operation (0 - read, 1 - write); 
	       data       - pointer to buffer where to put data;
	       datalen    - data length;
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               data buffer is allocated and released by user.
*/
int mlnx_lpc_rw(u16 base, u16 offset, u8 read_write, char *data, size_t datalen);

#endif /* __MLNX_IF_US_H_ */
